/**
 * chessboard.cpp
 *
 * Class representing the state of the game board
 *
 * Zach Kamsler
 * IMGD 4000
 */

#include "chessboard.h"
#include <cstring>
#include <cmath>
#include <algorithm>



/**
 * Constructs a chess board
 */
ChessBoard::ChessBoard() : whiteKing(7,4), blackKing(0,4), whiteCheck(false), blackCheck(false), turn(WHITE), canMove(true), turnCount(0), numMoves(0) {
	//clear the board
	memset(board, 0,  sizeof(board));

	//initialize white side
	initSide(WHITE);

	//initialize black side
	initSide(BLACK);
}

/**
 * Contructs a chess board from the given chess board
 *
 * @param o		the chess board to copy from
 */
ChessBoard::ChessBoard(const ChessBoard& o) : whiteKing(o.whiteKing), blackKing(o.blackKing), whiteCheck(o.whiteCheck), blackCheck(o.blackCheck), turn(o.turn), canMove(o.canMove), turnCount(o.turnCount), numMoves(o.numMoves) {
	memcpy(board, o.board, sizeof(board));
}

/**
 * Initializes the pieces of the given side.
 *
 * @param s		the side to initialize
 */
void ChessBoard::initSide(Side s) {
	int offset = (s == WHITE ? 0 : 7);

	// set up pawns
	int row = offset + 6*s;
	for(int i = 0; i < BOARDSIZE; i++) {
		board[row][i] = s*PAWN;
	}

	// set up rooks, knights, bishops
	row = offset + 7*s;
	board[row][0] = s*ROOK;
	board[row][7] = s*ROOK;
	board[row][1] = s*KNIGHT;
	board[row][6] = s*KNIGHT;
	board[row][2] = s*BISHOP;
	board[row][5] = s*BISHOP;

	// set up king, queen
	board[row][3] = s*QUEEN;
	board[row][4] = s*KING;
}

/**
 * Evaluates the current state of the board
 *
 * @param materialWeight	the weight for the material value of the board (pieces)
 * @param checkWeight		the weight for when a side is in check
 * @param mobilityWeight	the weight for piece mobility
 */
int ChessBoard::evaluate(int materialWeight, int checkWeight, int mobilityWeight) const {
	int value = 0;

	static const int PIECE_VALUES[] = {0,1,3,3,5,9,1000};

	// material value
	for(int row = 0; row < BOARDSIZE; row++) {
		for(int col = 0; col < BOARDSIZE; col++) {
			signed char p = getPiece(row, col);
			value += materialWeight*sideOf(p)*PIECE_VALUES[abs(p)];
		}
	}

	// king safety
	value += BLACK*whiteCheck*checkWeight;
	value += WHITE*blackCheck*checkWeight;

	// mobility
	//value += WHITE*mobilityWeight*calculateMobility(WHITE);
	//value += BLACK*mobilityWeight*calculateMobility(BLACK);

	// some randomness
	value += rand()&7;

	// include draws
	if(draw()) {
		value = 0;
	}

	// checkmate
	if(checkmate()) {
		value = reverseSide(turn)*(INFINITY-1);
	}

	return value*turn;
}

/**
 * Checks if the given row and column is on the board
 *
 * @param row	the row to check
 * @param col	the column to check
 *
 * @return true if the given row and column are on the board
 */
bool ChessBoard::onBoard(int row, int col) const {
	return (row >= 0) && (row < BOARDSIZE) && (col >= 0) && (col < BOARDSIZE);
}

/**
 * Checks if the given position is on the board
 *
 * @param pos	the position to check
 *
 * @return true if the given row and column are on the board
 */
bool ChessBoard::onBoard(const Position& pos) const {
	return onBoard(pos.row, pos.col);
}

/**
 * Checks if the given spot is empty
 *
 * @param row	the row to check
 * @param col	the column to check
 *
 * @return true if the given spot is empty
 */
bool ChessBoard::isEmpty(int row, int col) const {
	return onBoard(row,col) && getPiece(row,col) == 0;
}

/**
 * Checks if the given spot is empty
 *
 * @param pos	the position to check
 *
 * @return true if the given spot is empty
 */
bool ChessBoard::isEmpty(const Position& pos) const {
	return isEmpty(pos.row,pos.col);
}

/**
 * Checks if the given spot has a piece of the given side
 *
 * @param row	the row to be checked
 * @param col	the column to be checked
 *
 * @return true if there is a unit of the given side on the given spot
 */
bool ChessBoard::hasSide(int row, int col, Side side) const {
	return onBoard(row,col) && sameSide(side, getPiece(row,col));
}

/**
 * Checks if the given spot has a piece of the given side
 *
 * @param pos	the position to check
 *
 * @return true if there is a unit of the given side on the given spot
 */
bool ChessBoard::hasSide(const Position& pos, Side side) const {
	return hasSide(pos.row, pos.col, side);
}

/**
 * Checks if the given spot has a piece
 *
 * @param pos	the position to check
 *
 * @return true if there is a piece on the given spot
 */
bool ChessBoard::hasPiece(const Position& pos) const {
	return hasPiece(pos.row, pos.col);
}

/**
 * Checks if the given spot has a piece
 *
 * @param row	the row to be checked
 * @param col	the column to be checked
 *
 * @return true if there is a piece on the given spot
 */
bool ChessBoard::hasPiece(int row, int col) const {
	return getPiece(row, col) != 0;
}

/**
 * Moves the piece at the given location to the given destination
 *
 * @param move		the move to make
 */
void ChessBoard::makeMove(const Move& move) {
	makeMove(move.start, move.end);
}

/**
 * Moves the piece at the given location to the given destination
 *
 * @param start		the location of the piece
 * @param dest		the destination of the piece
 */
void ChessBoard::makeMove(const Position& start, const Position& dest, bool calc) {
	if(hasPiece(start) && onBoard(dest)) {
		signed char p = getPiece(start);

		//check for pawn promotion
		if(hasType(p, PAWN) && (dest.row == 0 || dest.row == 7)) {
			p = sideOf(p)*QUEEN;
		}

		//check for king movement
		if(p == BLACK*KING) {
			blackKing = dest;
		} else if (p == WHITE*KING) {
			whiteKing = dest;
		}

		//update turn counter
		if(hasPiece(dest) || hasType(p, PAWN)) {
			turnCount = 0;
		} else {
			turnCount++;
		}
		numMoves++;

		board[dest.row][dest.col] = p;
		board[start.row][start.col] = 0;

		// update check
		whiteCheck = attackable(whiteKing);
		blackCheck = attackable(blackKing);

		//change turn
		turn = reverseSide(turn);

		// update canmove
		if(calc) canMove = calculateCanMove();
	}
}

/**
 * Returns the value of the piece at the given row and column
 *
 * @param row	the row to check
 * @param col	the column to check
 *
 * @return  the piece at row, col if it is on the board, else 0
 */
signed char ChessBoard::getPiece(int row, int col) const {
	if (!onBoard(row, col)) return 0;

	return board[row][col];
}

/**
 * Returns the value of the piece at the given position
 *
 *  @param pos	the position to check
 *
 * @return  the piece at pos if it is on the board, else 0
 */
signed char ChessBoard::getPiece(const Position& pos) const {
	return getPiece(pos.row, pos.col);
}

/**
 * Finds the moves that the piece at the given position can make
 *
 * @param pos	the position of the piece to get the moves of
 * @param moves	the vector to output to
 *
 */
void ChessBoard::movesFor(const Position& pos, vector<Position>& moves) const {
	signed char piece = getPiece(pos);

	if(piece) {
		MOVES[abs(piece)-1]->generateMoves(pos, sideOf(piece), *this, moves);
		removeBadMoves(pos, moves);
	}
}

/**
 * Checks if the piece at the given spot can be attacked
 *
 * @param row	the column of the piece to check
 * @param col	the row of the piece to check
 *
 * @return true if the piece can be attacked
 */
bool ChessBoard::attackable(int row, int col) const {
	return attackable(Position(row, col));
}

/**
 * Checks if the piece at the given spot can be attacked
 *
 * @param pos	the position of the piece to check

 * @return true if the piece can be attacked
 */
bool ChessBoard::attackable(const Position& pos) const {
	if(!hasPiece(pos)) return false;

	Side attackers = reverseSide(sideOf(getPiece(pos)));

	for(int i = 0; i < BOARDSIZE; i++) {
		for(int j = 0; j < BOARDSIZE; j++) {
			Position p(i, j);
			signed char piece = getPiece(p);

			if(sameSide(piece, attackers) && canAttack(p, pos)) {
				return true;
			}
		}
	}

	return false;
}

/**
 * Generates the moves than can be made by the current side
 *
 * @param moves		the vector to output to
 */
void ChessBoard::generateMoves(vector<Move>& moves) const {
	generateMoves(turn, moves);
}

/**
 * Generates the moves than can be made by the current side
 *
 * @param s			the side to generate moves for
 * @param moves		the vector to output to
 */
void ChessBoard::generateMoves(Side s, vector<Move>& moves) const {
	if(draw()) return;
	
	vector<Position> dests;

	for(int i = 0; i < BOARDSIZE; i++) {
		for(int j = 0; j < BOARDSIZE; j++) {
			if(hasSide(i,j,s)) {
				Position start(i,j);
				dests.clear();
				movesFor(start, dests);

				for(unsigned k = 0; k < dests.size(); k++) {
					moves.push_back(Move(start,dests[k]));
				}
			}
		}
	}
}


/**
 * Checks if the given side in in check
 */
bool ChessBoard::inCheck(Side side) const {
	return side==WHITE ? whiteCheck : blackCheck;
}

/// struct for helping check checking
struct check_checker {
	check_checker(const ChessBoard& b, const Position& s, Side sid) : board(b), start(s), side(sid) {}
	bool operator()(Position n) {
		return board.causesCheck(start, n, side);
	}
	const ChessBoard& board;
	const Position& start;
	Side side;
};

/**
 * Removes moves that cause check from the vector
 *
 * @param p			the start position of the moves
 * @param moves		vector of the destinations of the moves
 */ 
void ChessBoard::removeBadMoves(const Position& p, vector<Position>& moves) const {
	vector<Position>::iterator new_end = remove_if(moves.begin(), moves.end(), check_checker(*this, p, sideOf(getPiece(p))));
	moves.erase(new_end, moves.end());
}

/**
 * Checks if the given move causes check
 *
 * @param piece		the position of the piece to move
 * @param dest		the destination of the piece
 * @param s			the side to check Check for
 *
 * @return true if the move causes check
 */
bool ChessBoard::causesCheck(const Position& piece, const Position& dest, Side s) const {
	ChessBoard test(*this);
	test.makeMove(piece, dest, false);

	return test.inCheck(s);
}

/**
 * 
 */
bool ChessBoard::canAttack(const Position& piece, const Position& dest) const {
	signed char p = getPiece(piece);

	return p &&	MOVES[abs(p)-1]->canMove(piece, dest, sideOf(p), *this);
}

/**
 * Calculates whether the current side can make a move
 *
 * @return true if the current side can make a move
 */
bool ChessBoard::calculateCanMove() const {
	vector<Position> m;
	for(int i = 0; i < BOARDSIZE; i++) {
		for(int j = 0; j < BOARDSIZE; j++) {
			if(sameSide(turn,getPiece(i,j))) {
				movesFor(Position(i,j),m);
				if(!m.empty()) return true;
			}
		}
	}

	return false;
}

/**
 * Calculates the number of mode that the given side can make
 *
 * @param s		the side to count the moves for
 *
 * @return the number of mode that the given side can make
 */
int ChessBoard::calculateMobility(Side s) const {
	vector<Move> m;
	generateMoves(s, m);

	return m.size();
}

const Moves* ChessBoard::MOVES[] = {
		&PAWN_MOVES,
		&KNIGHT_MOVES,
		&BISHOP_MOVES,
		&ROOK_MOVES,
		&QUEEN_MOVES,
		&KING_MOVES
};

const PawnMoves ChessBoard::PAWN_MOVES;
const KnightMoves ChessBoard::KNIGHT_MOVES;
const BishopMoves ChessBoard::BISHOP_MOVES;
const RookMoves ChessBoard::ROOK_MOVES;
const QueenMoves ChessBoard::QUEEN_MOVES;
const KingMoves ChessBoard::KING_MOVES;

