/**
 * chessboard.h
 *
 * Class representing the state of the game board
 *
 * Zach Kamsler
 * IMGD 4000
 */

#ifndef _CHESSBOARD_H_
#define _CHESSBOARD_H_

#include "enums.h"
#include "moves.h"

/// the chess board class
class ChessBoard {
public:
	ChessBoard();
	ChessBoard(const ChessBoard& other);
	
	int evaluate(int materialWeight = 10, int checkWeight=50, int mobilityWeight=1) const;

	void movesFor(const Position& pos, vector<Position>& moves) const;

	void generateMoves(Side s, vector<Move>& moves) const;
	void generateMoves(vector<Move>& moves) const;

	bool onBoard(const Position& pos) const;
	bool hasPiece(const Position& pos) const;
	bool isEmpty(const Position& pos) const;
	bool hasSide(const Position& pos, Side side) const;
	bool onBoard(int rol, int col) const;
	bool hasPiece(int row, int col) const;
	bool isEmpty(int row, int col) const;
	bool hasSide(int row, int col, Side side) const;
	bool attackable(int row, int col) const;
	bool attackable(const Position& pos) const;
	bool canAttack(const Position& piece, const Position& dest) const;
	bool inCheck(Side side) const;
	bool causesCheck(const Position& piece, const Position& dest, Side s) const;
	bool checkmate() const {return !canMove && inCheck(turn);}
	bool stalemate() const {return !canMove && !inCheck(turn);}
	bool draw() const {return turnCount >= 50 || stalemate();}
	bool done() const {return checkmate() || draw(); }
	bool inCheck() const {return inCheck(turn);}


	Side getTurn() const {return turn;}
	int getNumMoves() const {return numMoves;}

	void makeMove(const Move& move);
	void makeMove(const Position& start, const Position& dest, bool calc=true);

	signed char getPiece(int row, int col) const;
	signed char getPiece(const Position& pos) const;

private:
	/// the board representation
	signed char board[8][8];
	Position whiteKing;
	Position blackKing;
	bool whiteCheck, blackCheck;
	bool canMove;
	Side turn;
	int turnCount, numMoves;

	void initSide(Side side);
	void removeBadMoves(const Position& p, vector<Position>& moves) const;
	bool calculateCanMove() const;
	int calculateMobility(Side s) const;

	// move generators
	static const PawnMoves PAWN_MOVES;
	static const KnightMoves KNIGHT_MOVES;
	static const BishopMoves BISHOP_MOVES;
	static const RookMoves ROOK_MOVES;
	static const QueenMoves QUEEN_MOVES;
	static const KingMoves KING_MOVES;
	
	static const Moves* MOVES[];
};

#endif

