/**
 * chessinterface.cpp
 *
 * Class representing the user interface for the chess game
 *
 * Zach Kamsler
 * IMGD 4000
 */

#include <ClanLib/core.h>
#include <ClanLib/display.h>

#include "chessinterface.h"
#include "chessboard.h"


/**
 * Constructs a ChessInterface
 */
ChessInterface::ChessInterface(const ChessBoard& b) : board(b), showCursor(false) {
	CL_ResourceManager resources("resources.xml");

	// load background surface
	background = CL_Surface("board", &resources);

	// load white piece surfaces
	white[PAWN-1] = CL_Surface("white/pawn", &resources);
	white[KNIGHT-1] = CL_Surface("white/knight", &resources);
	white[BISHOP-1] = CL_Surface("white/bishop", &resources);
	white[ROOK-1] = CL_Surface("white/rook", &resources);
	white[QUEEN-1] = CL_Surface("white/queen", &resources);
	white[KING-1] = CL_Surface("white/king", &resources);

	// load black piece surfaces
	black[PAWN-1] = CL_Surface("black/pawn", &resources);
	black[KNIGHT-1] = CL_Surface("black/knight", &resources);
	black[BISHOP-1] = CL_Surface("black/bishop", &resources);
	black[ROOK-1] = CL_Surface("black/rook", &resources);
	black[QUEEN-1] = CL_Surface("black/queen", &resources);
	black[KING-1] = CL_Surface("black/king", &resources);

	// load cursor surface
	cursor = CL_Surface("cursor", &resources);

	// load highlight surfaces
	highlight = CL_Surface("highlight", &resources);
	highlightAttack = CL_Surface("highlight_attack", &resources);

	// load status
	bottom = CL_Surface("bottom", &resources);
	font = CL_Font("times new roman", 32);
	font.set_color(0,0,0);

	// connect mouse click handler
	mousePress = CL_Mouse::sig_key_down().connect(this, &ChessInterface::handleMouseClick);
}

/**
 * Draws the chess interface
 */
void ChessInterface::draw() {
	// draw the background
	background.draw();

	// draw the pieces
	for(int row = 0; row < BOARDSIZE; row++) {
		for(int col = 0; col < BOARDSIZE; col++) {
			signed char piece = board.getPiece(row, col);
			if(piece) {
				getSurfaceFor(piece).draw((float)col*TILESIZE,(float)row*TILESIZE);
			}
		}
	}

	// draw the cursor
	if(showCursor) {
		cursor.draw((float)cursorPos.col*TILESIZE, (float)cursorPos.row*TILESIZE);
	}

	// draw highlighted tiles
	for(unsigned int i = 0; i < highlighted.size(); i++) {
		Position& p = highlighted[i];

		if(board.hasPiece(p)) {
			highlightAttack.draw((float)p.col*TILESIZE, (float)p.row*TILESIZE);
		} else {
			highlight.draw((float)p.col*TILESIZE, (float)p.row*TILESIZE);
		}
	}

	// draw status
	bottom.draw(0, BOARDSIZE*TILESIZE);
	font.draw(16, BOARDSIZE*TILESIZE+16, board.getTurn() == WHITE ? "White Turn" : "Black Turn");
	if(board.checkmate()) {
		font.draw(380, BOARDSIZE*TILESIZE+16, "Checkmate");
	} else if(board.stalemate()) {
		font.draw(380, BOARDSIZE*TILESIZE+16, "Stalemate");
	} else if(board.draw()) {
		font.draw(380, BOARDSIZE*TILESIZE+16, "Draw");
	} else if(board.inCheck()) {
		font.draw(380, BOARDSIZE*TILESIZE+16, "In Check");
	} 

}

/**
 * Get the surface for the given piece
 */
CL_Surface& ChessInterface::getSurfaceFor(signed char piece) {
	if(isWhite(piece)) {
		return white[piece-1];
	} else {
		return black[-piece-1];
	}
}

/**
 * Sets the highlighted region of the board
 *
 * @param tiles the tile to highlight
 */
void ChessInterface::setHighlighted(const std::vector<Position>& tiles) {
	highlighted = tiles;
}

/**
 * Clears the highlighted region of the board
 */
void ChessInterface::clearHighlighted() {
	highlighted.clear();
}

/**
 * Sets the cursor to be at the given position.
 *
 * @param pos	the position to set the cursor to
 */
void ChessInterface::setCursor(const Position& pos) {
	showCursor = true;
	cursorPos = pos;
}


/**
 * Hides the cursor
 */
void ChessInterface::hideCursor() {
	showCursor = false;
}


/**
 * Responds to a mouse click event
 */
void ChessInterface::handleMouseClick(const CL_InputEvent &e) {
	selection = Position(e.mouse_pos.y / TILESIZE, e.mouse_pos.x / TILESIZE);
	selectionChanged.call(selection);
}
