/*
	Main application
*/

#include <ClanLib/core.h>
#include <ClanLib/application.h>
#include <ClanLib/gui.h>
#include <ClanLib/guistylesilver.h>
#include <ClanLib/display.h>
#include <ClanLib/network.h>
#include <ClanLib/gl.h>

#include "chessboard.h"
#include "chessinterface.h"
#include "humanplayer.h"
#include "chessclient.h"
#include "splashscreen.h"
#include "aiplayer.h"

class Application : public CL_ClanApplication
{
public:
	virtual int main(int, char **)
	{
		// Create a console window for text-output if not available
		//CL_ConsoleWindow console("Console");
		//console.redirect_stdio();

		// set up clanlib libraries
		CL_SetupCore setup_core;
		CL_SetupDisplay setup_display;
		CL_SetupGL setup_gl;
		CL_SetupNetwork setup_network;

		// create window
		CL_DisplayWindow window("Chess", 512, 576);
		CL_Slot slotQuit = window.sig_window_close().connect(this, Application::onClose);
		ok = true;

		// run splash screen
		background = CL_Surface("data/splash.png");
		SplashScreen splash(background);
		splash.run();


		CL_ResourceManager gui_resources("data/gui.xml");
		CL_StyleManager_Silver style(&gui_resources);
		CL_GUIManager gui(&style);
		
		// Set up startup menu
		CL_Slot slotRedraw = gui.sig_begin_paint().connect(this, Application::redraw);

		CL_Button run(CL_Rect(192,528,316,544), "Run", &gui, &style);
		CL_Slot runSlot = run.sig_clicked().connect(&gui, CL_GUIManager::quit);

		CL_Label whiteLabel(CL_Point(96, 386), "Player", &gui, &style);


		// Setup player ui
		CL_CheckBox ai(CL_Point(96, 408), "AI Player", &gui, &style);
		CL_Label skillLabel(CL_Point(96, 432), "Skill", &gui, &style);
		CL_RadioButton novice(CL_Point(104, 448), "Novice", &gui, &style);
		CL_RadioButton intermediate(CL_Point(104, 464), "Intermediate", &gui, &style);
		CL_RadioButton expert(CL_Point(104, 480), "Expert", &gui, &style);
		CL_CheckBox aggressive(CL_Point(96,512), "Aggressive", &gui, &style);
		CL_RadioGroup skill;
		skill.add(&novice);
		skill.add(&intermediate);
		skill.add(&expert);

		CL_Label hostLabel(CL_Point(304, 432), "Hostname", &gui, &style);
		CL_InputBox hostname(CL_Rect(304,448,400,464), "localhost", &gui, &style);


		// run gui
		gui.run();

		// set up board
		ChessBoard board;
		ChessInterface inter(board);
		this->inter = &inter;

		//initialize players
		Player* player = createPlayer(&ai, &novice, &intermediate, &expert, &aggressive);
		
		//set up chess game
		ChessClient game(board, *player);
		game.connect(hostname.get_text());

		// Loop until the user hits escape:
		inter.draw();
		while (ok && CL_Keyboard::get_keycode(CL_KEY_ESCAPE) == false)
		{
			game.update();
			inter.draw();

			cout << "flip\n";
			// Flip front and backbuffer. This makes the changes visible:
			CL_Display::flip();

			// Improve responsiveness:
			CL_System::sleep(10);
				
			// Update keyboard input and handle system events:
			CL_System::keep_alive();
		}

		cout << "closing\n";
		
		// delete players
		//delete player;

		exit(0);

		return 0;
	}

	/// whether to coninue the game loop
	bool ok;

	CL_Surface background;
	ChessInterface* inter;

	// AI skill levels map to maximum depths in the search
	const static int NOVICE = 2;
	const static int INTERMEDIATE = 4;
	const static int EXPERT = 10;

	/**
	 * Creates a player based on the given GUI elements
	 *
	 * @param ai			checkbox of whether to use AI
	 * @param novice		radio button for novice level AI
	 * @param intermediate	radio button for intermediate level AI
	 * @param expert		radio button for expert level AI
	 * @param aggressive	checkbox for aggressive AI
	 */
	Player* createPlayer(CL_CheckBox* ai, CL_RadioButton* novice, CL_RadioButton* intermediate, CL_RadioButton* expert, CL_CheckBox* aggressive) {
		if(ai->is_checked()) {
			int depth = NOVICE;

			// find difficulty level
			if(novice->is_checked()) {
				depth = NOVICE;
			} else if(intermediate->is_checked()) {
				depth = INTERMEDIATE;
			} else if(expert->is_checked()) {
				depth = EXPERT;
			}

			// return AI player
			return new AIPlayer(depth, aggressive->is_checked());
		}

		//return human player
		return new HumanPlayer(*inter);
	}

	void redraw() {
		background.draw();
	}
	void onClose() {
		ok = false;
	}

} app;
