/**
 * moves.cpp
 *
 * Classes for generating chess moves
 *
 * Zach Kamsler
 * IMGD 4000
 */

#include "moves.h"
#include "chessboard.h"


/**
 * Generates the moves for a knight
 *
 * @param pos	position of the piece on the board
 * @param side	the side of the piece
 * @param board	the board to generate moves for
 * @param moves	the vector to output moves to
 */
void KnightMoves::generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const {
	Side enemy = reverseSide(side);

	// for all possible moves
	for(int i = 0; i < numMoves; i++) {
		Position m = pos+moveOffsets[i];

		// if the square is empty or has an enemy, add it
		if(b.isEmpty(m) || b.hasSide(m, enemy)) {
			moves.push_back(m);
		}
	}
}

/**
 * Checks if a knight can make the given move
 *
 * @param pos	position of the piece on the board
 * @param move	position to check if it can be moved to
 * @param side	the side of the piece
 * @param board	the board to generate moves for
 *
 * @return true if the move is possible
 */
bool KnightMoves::canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const {
	Side enemy = reverseSide(side);

	// check if the destination is empty or has enemy
	if(!b.isEmpty(move) && !b.hasSide(move, enemy)) {
		return false;
	}

	// check if the move is one of the knight's moves
	for(int i = 0; i < numMoves; i++) {
		if(move == pos + moveOffsets[i]) {
			return true;
		}
	}

	// otherwise the move is not valid
	return false;
}

/// knightmoves
const Position KnightMoves::moveOffsets[] = {
	Position(2, 1),
	Position(1, 2),
	Position(-2, 1),
	Position(-1, 2),
	Position(2, -1),
	Position(1, -2),
	Position(-2, -1),
	Position(-1, -2)
};

const int KnightMoves::numMoves = 8;

/**
 * Generates the moves for a pawn
 *
 * @param pos	position of the piece on the board
 * @param side	the side of the piece
 * @param board	the board to generate moves for
 * @param moves	the vector to output moves to
 */
void PawnMoves::generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const {
	Side enemy = reverseSide(side);

	// check forwards
	Position m = pos + Position(-side,0);
	if(b.isEmpty(m)) {
		moves.push_back(m);

		// if on home row, and unoccupies double is ok
		m = m + Position(-side,0);
		if((pos.row == 1 || pos.row == 6) && b.isEmpty(m)) {
			moves.push_back(m);
		}
	}

	// check attacks
	m = pos + Position(-side, 1);
	if(b.hasSide(m, enemy)) {
		moves.push_back(m);
	}

	m = pos + Position(-side, -1);
	if(b.hasSide(m, enemy)) {
		moves.push_back(m);
	}
}

/**
 * Checks if a pawn can make the given move
 *
 * @param pos	position of the piece on the board
 * @param move	position to check if it can be moved to
 * @param side	the side of the piece
 * @param board	the board to generate moves for
 *
 * @return true if the move is possible
 */
bool PawnMoves::canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const {
	Side enemy = reverseSide(side);

	// check forwards
	Position m = pos + Position(-side,0);
	if(b.isEmpty(move)) {
		if(move == m) {
			return true;
		}

		// if on home row, and unoccupies double is ok
		if((pos.row == 1 || pos.row == 6) && b.isEmpty(m) && move == m+Position(-side,0)) {
			return true;
		}
	}

	// check attacks
	Position m1 = pos + Position(-side, 1);
	Position m2 = pos + Position(-side, -1);
	if(b.hasSide(move, enemy) && (move==m1 || move==m2)) {
		return true;
	}

	
	return false;
}

/**
 * Generates the moves for a piece
 *
 * @param pos	position of the piece on the board
 * @param side	the side of the piece
 * @param board	the board to generate moves for
 * @param moves	the vector to output moves to
 */
void DirectionalMoves::generateMoves(const Position& pos, Side side, const ChessBoard& b, vector<Position>& moves) const {
	Side enemy = reverseSide(side);

	for(unsigned int i = 0; i < directions.size(); i++) {
		Position m = pos;

		for(int j = 0; j < distance; j++) {
			m = m + directions[i];

			if(b.isEmpty(m)) {
				moves.push_back(m);
			} else {
				if(b.hasSide(m, enemy)) moves.push_back(m);
				break;
			}
		}
	}
}

/**
 * Checks if a piece can make the given move
 *
 * @param pos	position of the piece on the board
 * @param move	position to check if it can be moved to
 * @param side	the side of the piece
 * @param board	the board to generate moves for
 *
 * @return true if the move is possible
 */
bool DirectionalMoves::canMove(const Position& pos, const Position& move, Side side, const ChessBoard& b) const {
	Side enemy = reverseSide(side);

	for(unsigned int i = 0; i < directions.size(); i++) {
		Position m = pos;

		for(int j = 0; j < distance; j++) {
			m = m + directions[i];

			if(m == move) {
				return b.isEmpty(m) || b.hasSide(m, enemy);
			} else if(!b.isEmpty(m)) {
				break;
			}
		}
	}

	return false;
}


/**
 * Constructs a KingMoves
 */
KingMoves::KingMoves() : DirectionalMoves() {
	distance = 1;
	directions.push_back(Position(1,0));
	directions.push_back(Position(0,1));
	directions.push_back(Position(-1,0));
	directions.push_back(Position(0,-1));
	directions.push_back(Position(1,1));
	directions.push_back(Position(-1,1));
	directions.push_back(Position(1,-1));
	directions.push_back(Position(-1,-1));
}


/**
 * Constructs a QueenMoves
 */
QueenMoves::QueenMoves() : DirectionalMoves() {
	directions.push_back(Position(1,0));
	directions.push_back(Position(0,1));
	directions.push_back(Position(-1,0));
	directions.push_back(Position(0,-1));
	directions.push_back(Position(1,1));
	directions.push_back(Position(-1,1));
	directions.push_back(Position(1,-1));
	directions.push_back(Position(-1,-1));
}


/**
 * Constructs a RookMoves
 */
RookMoves::RookMoves() : DirectionalMoves() {
	directions.push_back(Position(1,0));
	directions.push_back(Position(0,1));
	directions.push_back(Position(-1,0));
	directions.push_back(Position(0,-1));
}

/**
 * Constructs a BishopMoves
 */
BishopMoves::BishopMoves() : DirectionalMoves() {
	directions.push_back(Position(1,1));
	directions.push_back(Position(-1,1));
	directions.push_back(Position(1,-1));
	directions.push_back(Position(-1,-1));
}
